/**
 * @module file.js
 * @desc file select.
 * @param id
 * @param option
 * @returns {*}
 */

'use strict';

import React        from 'react'
import ReactDOM     from 'react-dom'
import * as alert   from '../core/alert'
import * as net     from '../core/net'
import * as util    from '../core/util'

export default class FileSelect extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      value: this.array(this.props.value),
      text: this.array(this.props.text),
      data: this.props.data,
      accept: this.props.accept,
      multiple: this.props.multiple,
      size: this.props.size,
      className: Object.assign({}, FileSelect.defaultProps.className, this.props.className)
    };
    this.click = this.click.bind(this);
    this.onFileClick = this.onFileClick.bind(this);
    this.onFileUpload = this.onFileUpload.bind(this);
    this.onTokenFileUpload = this.onTokenFileUpload.bind(this);
  }

  render() {
    if (this.props.ui === 'none' || this.props.ui === 'bind') {
      return this.renderInput();
    }

    return dom('div', {className: this.state.className.base},
      this.renderButton('left'),
      dom('input', {
        type: 'text',
        className: this.state.className.input,
        readOnly: true,
        style: {backgroundColor: '#fff'},
        value: this.state.text.join(','),
        onClick: this.onFileClick
      }),
      this.renderInput(),
      this.renderButton('right')
    );
  }

  renderButton(side) {
    if (side === this.props.button) {
      return dom('span', {className: 'input-group-btn', style: {backgroundColor: '#fff'}},
        dom('button', {className: 'btn btn-default', type: 'button', onClick: this.click},
          dom('i', {className: this.props.buttonIcon}),
          dom('span', {}, this.props.buttonText)
        )
      );
    }

    return undefined;
  }

  renderInput() {

    return dom('input', {
      style: {display: 'none'},
      accept: this.state.accept,
      multiple: this.state.multiple,
      onChange: this.props.urltoken ? this.onTokenFileUpload : this.onFileUpload,
      type: 'file',
      ref: 'file'
    });
  }

  onFileClick() {
    if (!this.props.urldownload) {
      return;
    }

    if (this.state.value.length <= 0) {
      return;
    }

    FileSelect.download(this.props.urldownload + this.state.value[0]);
  }

  onFileUpload(event) {
    const files = event.target.files;

    // No files selected
    if (!files || files.length <= 0) {
      return false;
    }

    // Create form data
    let fd = new FormData(), size = 0;
    for (let i = 0; i < files.length; i++) {
      let file = files[i];

      size += file.size;
      fd.append('files', file);
    }

    // Check file size
    if (size > this.state.size) {
      return this.props.error('The file is too large');
    }

    // Upload file
    net.postData(this.props.url, this.state.data, fd, (err, result) => {
      if (err) {
        return this.props.error(err, result);
      }

      if (this.props.ui === 'selector') {
        this.setState({text: result.map(item => item.name), value: result.map(item => item._id)});
      }

      this.props.success(result);
    }, this.props.progress);

    // Clear input value （In order to accept another change event）
    this.refs.file.value = '';
  }

  onTokenFileUpload(event) {
    const files = event.target.files;

    // No files selected
    if (!files || files.length <= 0) {
      return false;
    }

    let hasSizeError = false;

    // Create form data, 支持多文件上传
    const data = _.map(files, file => {
      const fd = new FormData();
      fd.append('file', file);
      if (file.size > this.state.size) {
        hasSizeError = true;
      }
      return {name: file.name, fd: fd};
    });

    // Check file size
    if (hasSizeError) {
      return this.props.error('The file is too large');
    }

    // 如果上传文件时需要token认证，就先调用tokenAPI，然后上传
    net.get(this.props.urltoken, (err, result) => {
      if (err) {
        return this.props.error(err, result);
      }

      this.uploadMulti(data, result.token, result => {
        if (this.props.ui === 'selector') {
          this.setState({text: result.map(item => item.filename), value: result.map(item => item._id)});
        }

        this.props.success(result);
      });
    });

    // Clear input value （In order to accept another change event）
    this.refs.file.value = '';
  }

  uploadMulti(data, token, callback) {

    let files = [];

    data.forEach(item => {

      const key = util.guid8() + util.guid8() + util.guid8();
      item.fd.append('key', key);
      item.fd.append('token', token);

      net.postData(this.props.url, this.state.data, item.fd, (err, result) => {
        if (err) {
          return this.props.error(err, result);
        }

        // 保存上传结果
        files.push({filename: item.name, _id: key});

        // 执行到最后一条数据，返回
        if (files.length >= data.length) {
          return callback(files);
        }
      }, this.props.progress);
    });
  }

  componentDidMount() {

    // 如果是bing类型，需要将上传事件绑定到父组件的单击事件上
    if (this.props.ui === 'bind') {

      let owner = ReactDOM.findDOMNode(this).parentNode.parentNode;
      owner.addEventListener('click', () => {
        let clickEvent = document.createEvent('MouseEvents');
        clickEvent.initEvent('click', false, false);
        ReactDOM.findDOMNode(this.refs.file).dispatchEvent(clickEvent);
      });
    }
  }

  componentDidUpdate(prevProps) {

    const state = {};

    if (prevProps.accept !== this.props.accept) {
      state.accept = this.props.accept;
    }
    if (prevProps.multiple !== this.props.multiple) {
      state.multiple = this.props.multiple;
    }
    if (prevProps.size !== this.props.size) {
      state.size = this.props.size;
    }
    if (!_.isEqual(this.array(prevProps.value), this.array(this.props.value))) {
      state.value = this.array(this.props.value);
    }
    if (!_.isEqual(this.array(prevProps.text), this.array(this.props.text))) {
      state.text = this.array(this.props.text);
    }

    if (Object.keys(state).length > 0) {
      this.setState(state);
    }
  }

  array(val) {
    if (!val) {
      return [];
    }

    return Array.isArray(val) ? val : [val];
  }

  click() {
    $(ReactDOM.findDOMNode(this.refs.file)).trigger('click');
  }

  upload(data) {

    if (data) {
      this.state.data = data;
    }
    this.click();
  }

  download(data) {
    FileSelect.download(this.props.url, data);
  }

  getValue() {
    return this.state.value;
  }
}

FileSelect.defaultProps = {
  accept: '*',
  button: 'right',
  size: '5242880',
  nometa: false,
  multiple: false,
  ui: 'none',   // bind - 将上传功能绑定到指定的按钮上，id参数需要指定按钮的id
                // selector - 带有一个输入框和按钮的上传组件
                // none - 无UI，只有上传功能，需要自行调用upload方法来启动上传操作
  url: '/api/file/add',
  urlmeta: '/api/file/upload',
  urltoken: '', // 获取Token的URL
  urldownload: '',
  data: {},     // 上传时的附属信息
  buttonText: ' Choose',
  buttonIcon: ' fa fa-folder-open',
  className: {
    base: 'input-group',
    input: '',
    button: 'input-group-btn'
  },
  value: [],    // 文件ID
  text: [],     // 文件名称
  check: () => {
  },
  success: () => {
  },
  progress: val => {
  },
  error: (msg) => {
    alert.error(msg || 'Upload file error');
  }
};

FileSelect.download = (url, param) => {
  window.location = param ? url + "?" + $.param(param) : url;
};

FileSelect.init = (id, option) => {

  if (_.isUndefined(option)) {
    option = id;
    id = undefined;
  }

  // 没有指定ID，在body里添加一个上传组件
  if (!id) {

    id = "file-element";

    let container = $('body');
    if (!container.has($('#' + id)).length) {
      container.prepend(`<div id="${id}"></div>`);
    }
  }

  // 类型为bing时，创建一个上传组件
  if (option && option.ui === 'bind') {

    let container = $('#' + id);
    if (!container.has($(`#${id}-element`)).length) {
      container.prepend(`<div id="${id}-element"></div>`);
    }
    id = id + '-element';
  }

  return ReactDOM.render(
    React.createElement(FileSelect, option),
    document.getElementById(id)
  );
};
