/**
 * @module Pagination
 * @desc pagination.
 * @param id
 * @param option
 * @param {number} option.rowCount - row counts of every page
 * @param {number} option.totalItems - total items
 * @param {string} option.type - [ pages | more ]
 * @param {function} option.fetchFunction
 * @param {function} option.fetchOption
 * @param {function} option.className
 * @returns {*}
 */
'use strict';

import React    from 'react'
import ReactDOM from 'react-dom'
import CONST    from '../constant'


export default class Pagination extends React.Component {

  /**
   * constructor
   * @param props
   */
  constructor(props) {
    super(props);
    this.state = this.initState();
    this.changePage = this.changePage.bind(this);
    this.onMoreClick = this.onMoreClick.bind(this);
  }

  initState(totalItems) {

    let defaultClassName = Pagination.defaultProps.className;

    this.totalPages = Math.ceil((totalItems || this.props.totalItems) / this.props.rowCount);
    return {
      limitPage: this.totalPages > this.props.pageCount ? this.props.pageCount : this.totalPages,
      startPage: 1,
      activePage: 1,
      rowCount: this.props.rowCount,
      totalItems: totalItems,
      className: _.extend(defaultClassName, this.props.className)
    };
  }

  reset(fetchOption) {
    this.props.fetchOption.condition = (fetchOption || this.props.fetchOption).condition;
    this.action = undefined;
    this.fetch(0);
  }

  update(totalItems) {

    // first call
    if (!this.action) {
      this.setState(this.initState(totalItems));
    }

    // prev
    else if (this.action === CONST.PAGE_PREV) {
      this.action = this.state.startPage - this.props.pageCount < 1 ? 1 : this.state.startPage - this.props.pageCount;
      this.setState({
        startPage: this.action,
        activePage: this.action,
        limitPage: this.props.pageCount
      });
    }

    // next
    else if (this.action === CONST.PAGE_NEXT) {
      let loadedItems = (this.state.startPage - 1) * this.state.rowCount
        , remainder = Math.ceil((this.state.totalItems - loadedItems) / this.state.rowCount) - this.props.pageCount;

      this.action = this.state.startPage + this.props.pageCount;
      this.setState({
        startPage: this.action,
        activePage: this.action,
        limitPage: remainder < this.props.pageCount ? remainder : this.props.pageCount
      });
    }

    // number button
    else if (this.action !== this.state.activePage) {
      this.setState({activePage: this.action, totalItems: totalItems});
    }

    // more button
    else if (this.action === CONST.PAGE_MORE) {
      this.setState({startPage: this.state.startPage, totalItems: totalItems});
    }
  }

  render() {
    if (this.state.totalItems < this.props.rowCount) {
      return null;
    }
    return dom('div', {className: this.state.className.base}, this.renderPages(), this.renderMore())
  }

  renderPages() {
    if (this.props.type !== CONST.PAGE_TYPE_PAGES) {
      return null;
    }

    return dom('div', {className: 'btn-group'},
      this.prev(),
      this.page(),
      this.next()
    );
  }

  renderMore() {
    if (this.props.type !== CONST.PAGE_TYPE_MORE) {
      return null;
    }

    if (this.state.totalItems < this.state.startPage + this.state.rowCount) {
      return null;
    }

    return dom('button', {className: this.state.className.baseMore, onClick: this.onMoreClick},
      dom('i', {className: this.state.className.moreIcon}), this.props.moreButtonText
    );
  }

  page() {
    return _.range(this.state.startPage, this.state.startPage + this.state.limitPage).map(i => {

      let className = this.state.className.baseButton + ' ' + (i === this.state.activePage
          ? this.state.className.active : this.state.className.default);

      return dom('a', {key: i, className: className, 'data-page': i, onClick: this.changePage}, i);
    });
  }

  prev() {
    return dom('a', {
        'data-page': CONST.PAGE_PREV,
        className: this.state.className.baseButton + ' ' + this.state.className.default,
        onClick: this.changePage,
        disabled: (this.state.startPage > 1 ? null : 'disabled'),
      },
      dom('i', {className: this.state.className.arrowLeft})
    );
  }

  next() {
    let disabled = (this.state.startPage + this.state.limitPage - 1 < this.totalPages) ? null : 'disabled';

    return dom('a', {
        'data-page': CONST.PAGE_NEXT,
        className: this.state.className.baseButton + ' ' + this.state.className.default,
        onClick: this.changePage,
        disabled: disabled
      },
      dom('i', {className: this.state.className.arrowRight})
    );
  }

  changePage(event) {

    if (event.currentTarget.hasAttribute('disabled')) {
      return false;
    }

    this.action = parseInt(event.currentTarget.getAttribute('data-page'));
    if (this.action === CONST.PAGE_PREV) {
      let prevStart = this.state.startPage - this.props.pageCount < 1 ? 1 : this.state.startPage - this.props.pageCount;
      this.fetch((prevStart - 1) * this.state.rowCount);
      return false;
    }

    if (this.action === CONST.PAGE_NEXT) {
      let nextStart = this.state.startPage + this.props.pageCount;
      this.fetch((nextStart - 1) * this.state.rowCount);
      return false;
    }

    if (this.action !== this.state.activePage) {
      this.fetch((this.action - 1) * this.state.rowCount);
    }
  }

  onMoreClick() {
    this.action = CONST.PAGE_MORE;
    this.state.startPage += this.state.rowCount;
    this.fetch(this.state.startPage);
  }

  /**
   * fetch the data on each page
   * @param skip
   */
  fetch(skip) {
    this.props.fetchOption.skip = skip;
    this.props.fetchFunction(this.props.fetchOption);
  }
}

Pagination.defaultProps = {
  fetchOption: {skip: 0, limit: CONST.PAGE_ROW_COUNT},
  fetchFunction: () => {
  },

  totalItems: 0,
  rowCount: CONST.PAGE_ROW_COUNT,
  pageCount: CONST.PAGE_PAGE_COUNT,
  type: CONST.PAGE_TYPE_PAGES,
  moreButtonText: ' 加载更多',

  className: {
    base: null,
    baseButton: 'btn',
    baseMore: 'btn btn-block btn-outline btn-default',
    moreIcon: 'fa fa-arrow-circle-down',
    active: 'btn-info',
    default: 'btn-default',
    arrowLeft: 'fa fa-angle-left',
    arrowRight: 'fa fa-angle-right'
  }
};

Pagination.init = (id, option) => {
  return ReactDOM.render(
    React.createElement(Pagination, option),
    document.getElementById(id)
  );
};
