/**
 * @module SelectBox
 * @desc Depend.
 * <ul>
 *   <li>react</li>
 *   <li>bootstrap</li>
 *   <li>fontawesome</li>
 * </ul>
 *
 * React Element Structure:
 *   modal
 *    modal-dialog
 *      modal-content
 *        modal-header
 *          title & search
 *        modal-body
 *          grid
 *        modal-footer
 *          pagination
 *
 * @param id
 * @param option
 * @param {string} option.type - 预设类型: user group category authority role tag route function board structure
 * @param {string} option.api - 检索数据用API
 * @param {object} option.condition - 检索条件
 * @param {array} option.value - 选中的项目, 如果指定了uk, 那么value应该是uk值列表
 * @param {string} option.uk - 识别选中用的字段名称 default: _id
 * @param {string} option.name - 确定按钮时, 可以返回选中项目的name
 * @param {array} option.field - 显示的列名称 default: ['_id']
 * @param {boolean} option.single - 只允许选择一个
 * @param {string} option.icon - 显示的图标
 * @param {boolean} option.search - 是否显示检索按钮 default: true
 * @param {boolean} option.pagination - 是否显示翻页按钮 default: true
 * @param {int} option.show - 一页显示的行数 default: 10
 * @param {array} option.data - 允许不使用api, 直接提供数据显示选择框 - TODO
 * @param {function} option.ok - done event
 * @param {string} option.title - title
 * @returns {*}
 */

'use strict';

import React      from 'react'
import ReactDOM   from 'react-dom'
import CONST      from '../constant';
import * as net   from '../core/net'
import * as util  from '../core/util'


export default class SelectBox extends React.Component {

  constructor(props) {
    super(props);

    // 初始化已经被选中的项目名称（翻页后进行选择时，需要记住选中项目名称而不是再去数据库中取）
    this.names = this.props.names || [];
    this.emitter = this.props.emitter || $({});

    this.emit = this.emit.bind(this);
    this.ok = this.ok.bind(this);
    this.cancel = this.cancel.bind(this);
    this.change = this.change.bind(this);
    this.search = this.search.bind(this);
    this.prev = this.prev.bind(this);
    this.next = this.next.bind(this);

    this.emitter.on(CONST.EVENT.CHANGE, this.change);
    this.emitter.on(CONST.EVENT.SEARCH, this.search);
    this.emitter.on(CONST.EVENT.PREV, this.prev);
    this.emitter.on(CONST.EVENT.NEXT, this.next);
    this.emitter.on(CONST.EVENT.OK, this.ok);
    this.emitter.on(CONST.EVENT.CANCEL, this.cancel);

    const reserve = this.props.type ? CONST.RESERVE[this.props.type] : undefined;

    this.state = {
      api: reserve ? reserve.api : this.props.api,
      dataUk: reserve ? reserve.uk : this.props.dataUk,
      dataName: reserve ? reserve.name : this.props.dataName,
      icon: reserve ? reserve.icon : this.props.icon,
      field: reserve ? reserve.field : this.props.field,
      title: reserve ? reserve.title : this.props.title,

      data: this.props.data,
      value: this.props.value,
      show: this.props.show,
      single: this.props.single,
      search: this.props.search,
      pagination: this.props.pagination,
      emit: this.emit,
      total: 0,
      page: 0,
      condition: this.props.condition,
      style: this.props.style,

      hasParentDialog: false // 在弹出对话框中显示选择对话框时，需要调整margin，否则页面会乱。这里的style的Margin的影响需要确认。
    };
  }

  render() {

    return dom('div', {
        className: 'modal fade',
        style: this.state.hasParentDialog ? {marginBottom: -35} : null
      },
      dom('div', {
          className: 'modal-dialog',
          style: this.state.hasParentDialog ? {marginTop: 0, marginBottom: 0} : null
        },
        dom('div', {className: 'modal-content'},
          React.createElement(Header, this.state),
          React.createElement(Body, this.state),
          React.createElement(Footer, this.state)
        )
      )
    );
  }

  componentDidMount() {
    this.setState({hasParentDialog: $(ReactDOM.findDOMNode(this)).parents('.modal-dialog').length > 0});
  }

  componentDidUpdate(prevProps) {

    if (prevProps.type !== this.props.type) {
      const reserve = CONST.RESERVE[this.props.type];
      if (reserve) {
        this.setState({
          api: reserve.api,
          dataUk: reserve.uk,
          dataName: reserve.name,
          icon: reserve.icon,
          field: reserve.field,
          title: reserve.title
        });
      }
    }

    if (prevProps.api !== this.props.api) {
      this.setState({
        api: this.props.api,
        dataUk: this.props.dataUk,
        dataName: this.props.dataName,
        icon: this.props.icon,
        field: this.props.field,
        title: this.props.title,
      });
    }

    if (!_.isEqual(prevProps.value, this.props.value)) {
      this.state.value = this.props.value;
      this.setState({value: this.state.value});
    }

    if (!_.isEqual(prevProps.names, this.props.names)) {
      this.names = this.props.names;
    }
  }

  emit(type, data, node) {
    this.emitter.trigger(type, [data, node]);
  }

  /**
   * selection change event
   * @param event
   * @param value
   */
  change(event, value) {

    const item = this.state.data.find(item => item[this.state.dataUk] === value)
      , name = this.state.dataName ? item[this.state.dataName] : value;

    if (this.state.single) {

      this.state.value = [value];
      this.names = [name];
      return this.setState({value: this.state.value});
    }

    // 多选，没有被选中则添加到选中列表中
    if (this.state.value.indexOf(value) < 0) {

      this.names.push(name);
      this.state.value.push(value);
    } else {

      // 已经被选中，从列表中删除
      const nameIndex = this.names.findIndex(item => item === name);
      this.names.splice(nameIndex, 1);

      const valueIndex = this.state.value.findIndex(item => item === value);
      this.state.value.splice(valueIndex, 1);
    }

    this.setState({value: this.state.value});
  }

  /**
   * search event
   * @param event
   * @param keyword
   */
  search(event, keyword) {
    this.state.condition.condition['keyword'] = keyword;
    this.show();
  }

  /**
   * go to the next page
   */
  next() {
    this.state.page = this.state.page + 1;
    this.setState({page: this.state.page});
    this.show();
  }

  /**
   * go to the previous page
   */
  prev() {
    this.state.page = this.state.page - 1;
    this.setState({page: this.state.page});
    this.show();
  }

  /**
   * done event
   */
  ok() {
    $(ReactDOM.findDOMNode(this)).modal('hide');
    this.props.ok(this);
  }

  /**
   * cancel event
   */
  cancel() {
    $(ReactDOM.findDOMNode(this)).modal('hide');
    this.props.cancel(this);
  }

  show() {
    let condition = {
      skip: this.state.page * this.state.show,
      limit: this.state.show,
      condition: this.state.condition.condition
    };

    this.fetch(condition, data => {
      this.setState({data: data.items, total: data.totalItems});
      $(ReactDOM.findDOMNode(this)).modal('show');
    });

    return this;
  }

  fetch(condition, callback) {

    Object.assign(condition.condition, util.params(this.state.api));

    // 如果URL里指定了条件，转换为condition
    net.get(this.state.api.split('?')[0], condition, (err, result) => {
      if (err) {
        return console.log(err);
      }

      callback(result);
    });
  }

  /**
   * getValue - 选中的项目, 如果指定了uk, 那么value应该是uk值列表
   * @returns {*}
   */
  getValue() {
    return this.state.value;
  }

  /**
   * 修改选中项目
   * @param val
   */
  setValue(val) {
    this.setState({value: val});
  }

  /**
   * getName - 选中的项目name
   * @returns {*}
   */
  getName() {
    return this.names;
  }

  /**
   * getData - 选中项目的所有数据
   * @returns {Array}
   */
  getData() {
    let result = [];
    this.state.data.forEach(item => {
      if (this.state.value.indexOf(item[this.state.dataUk]) >= 0) {
        result.push(item);
      }
    });

    return result;
  }

}


class Header extends React.Component {
  constructor(props) {
    super(props);

    this.onClick = this.onClick.bind(this);
    this.onChange = this.onChange.bind(this);
    this.onKeyDown = this.onKeyDown.bind(this);

    this.state = {
      searching: false,
      keyword: ''
    };
  }

  render() {

    return dom('div', {className: 'modal-header', style: this.props.style.header},
      dom('section', {className: 'selectbox-search'},
        this.props.search && this.state.searching ?
          dom('input', {
            className: 'form-control',
            ref: 'input',
            onChange: this.onChange,
            onKeyDown: this.onKeyDown,
            value: this.state.keyword
          }) : null,
        this.props.search ?
          dom('a', {onClick: this.onClick}, dom('i', {className: this.icon()})) : null,
        this.state.searching ? null :
          dom('h4', {className: 'modal-title', style: this.props.style.title}, this.props.title)
      )
    );
  }

  icon() {
    return this.state.searching ? 'fa fa-close' : 'fa fa-search';
  }

  onChange(event) {
    this.setState({keyword: event.target.value});
  }

  onKeyDown(event) {
    if (event.keyCode === CONST.KEY.ENTER) {
      event.preventDefault();
      this.props.emit(CONST.EVENT.SEARCH, this.state.keyword);
    }
  }

  onClick() {
    if (this.state.searching) {
      this.props.emit(CONST.EVENT.SEARCH);
      return this.setState({
        searching: false,
        keyword: ''
      });
    }

    this.setState({
      searching: true,
      title: {visibility: 'hidden'}
    });
  }
}


class Body extends React.Component {

  constructor(props) {
    super(props);

    this.click = this.click.bind(this);
  }

  render() {
    return dom('div', {className: 'modal-body', style: this.props.style.body},
      dom('table', {className: 'table table-hover table-striped'},
        dom('tbody', {}, this.row())
      )
    );
  }

  row() {
    if (!this.props.data) {
      return undefined;
    }

    return this.props.data.map(function (row, i) {
      return dom('tr', {key: i}, this.icon(row), this.col(row), this.check(row));
    }.bind(this))
  }

  icon(row) {
    if (!this.props.icon) {
      return undefined;
    }

    let uk = row[this.props.dataUk];
    return dom('td', {style: {width: '5%', border: '0'}, value: uk, onClick: this.click},
      dom('i', {className: 'fa fa-' + this.props.icon, value: uk})
    );
  }

  col(row) {
    return this.props.field.map(function (col, i) {
      return dom('td', {style: {border: '0'}, value: row[this.props.dataUk], onClick: this.click, key: i}, row[col])
    }.bind(this));
  }

  check(row) {

    let uk = row[this.props.dataUk]
      , checked = this.props.value.indexOf(uk) >= 0;

    return dom('td', {style: {width: '6%', border: '0'}, value: uk, onClick: this.click},
      dom('i', {className: checked ? 'fa fa-check-square-o' : 'fa fa fa-square-o', value: uk})
    );
  }

  click(event) {
    this.props.emit(CONST.EVENT.CHANGE, event.target.getAttribute('value'));
  }
}


class Footer extends React.Component {

  constructor(props) {
    super(props);

    this.ok = this.ok.bind(this);
    this.cancel = this.cancel.bind(this);
    this.prev = this.prev.bind(this);
    this.next = this.next.bind(this);
  }

  render() {

    return dom('div', {className: 'modal-footer', style: this.props.style.footer},
      this.pagination(),
      dom('button', {type: 'button', className: 'btn btn-default btn-sm', onClick: this.cancel}, '取消'),
      dom('button', {type: 'button', className: 'btn btn-primary btn-sm', onClick: this.ok}, '确定')
    );
  }

  pagination() {
    if (!this.props.pagination) {
      return undefined;
    }

    let isFirst = this.props.page === 0
      , isLast = this.props.page === Math.floor(this.props.total / this.props.show);

    return dom('div', {style: {float: 'left'}},
      dom('button', {type: 'button', className: 'btn btn-default btn-sm', onClick: this.prev, disabled: isFirst},
        dom('i', {className: 'fa fa-caret-left'})
      ),
      dom('button', {type: 'button', className: 'btn btn-default btn-sm', onClick: this.next, disabled: isLast},
        dom('i', {className: 'fa fa-caret-right'})
      ),
      this.number()
    );
  }

  number() {
    let start = this.props.page * this.props.show + 1
      , end = this.props.page * this.props.show + this.props.show;

    end = end > this.props.total ? this.props.total : end;
    return dom('span', {style: {color: '#ccc', fontSize: '10px'}},
      '（' + start + ' ~ ' + end + ' / ' + this.props.total + '）'
    );
  }

  prev() {
    this.props.emit(CONST.EVENT.PREV);
  }

  next() {
    this.props.emit(CONST.EVENT.NEXT);
  }

  ok() {
    this.props.emit(CONST.EVENT.OK);
  }

  cancel() {
    this.props.emit(CONST.EVENT.CANCEL);
  }
}


SelectBox.defaultProps = {
  value: [],
  names: [],
  dataUk: '_id',              // 数据库中 ID字段名
  dataName: 'name',           // 数据库中 标题字段名
  dataKeys: 'keys',           // 检索API 获取名称条件 （$in） 初期显示被选中的项目名时使用
  keyword: 'keyword',         // 检索API 模糊检索添加条件 （$regexp）
  field: ['_id', 'name'],     // 显示的字段
  title: 'Select',
  show: 10,
  emitter: undefined,
  search: true,
  pagination: true,
  condition: {condition: {}},
  style: {      // 可以通过以下项目，设定元素的style，通过css难以控制时可以使用
    header: {},
    body: {},
    footer: {},
    title: {}
  },
  ok: () => {
  },
  cancel: () => {
  }
};


SelectBox.init = (id, option) => {
  return ReactDOM.render(
    React.createElement(SelectBox, option),
    document.getElementById(id)
  );
};
