/**
 * @module Tree
 * @desc tree.
 * @param id
 * @param option
 * @param {number} option.width
 * @param {boolean} option.wholerow
 * @param {boolean} option.checkbox
 * @param {string | object} option.icon - exp. icon: 'glyphicon glyphicon-folder-open'
 *                      exp. icon: 'fa fa-folder-o'
 *                      exp. icon: './static/images/tree_icon.png'
 * @param {array} option.data - data json format.
 * @param {string} option.data.id - required
 * @param {string} option.data.parent - required
 * @param {string} option.data.text - node text
 * @param {string} option.data.icon - string for custom icon.
 * @param {function} option.onChange
 * @param {function} option.onLoaded
 * @param {boolean} option.state.opened
 * @param {boolean} option.state.disabled
 * @param {boolean} option.state.selected
 * @param {string} option.api - TODO:
 * @param {object} option.condition - TODO:
 * @param {string} option.id - TODO:
 * @param {string} option.text - TODO:
 * @returns {*}
 */

'use strict';

import React    from 'react'
import ReactDOM from 'react-dom'
// import jstree   from 'jstree'


export default class Tree extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      data: this.props.data
    }
  }

  render() {
    return dom('div', {style: {width: this.props.width}});
  }

  componentDidMount() {

    // set tree options
    let plugins = ['types', 'changed', 'search'];
    if (this.props.wholerow) {
      plugins.push('wholerow');
    }
    if (this.props.checkbox) {
      plugins.push('checkbox');
    }

    // set icons
    let icons = {default: {icon: 'fa fa-folder-o'}};
    if (typeof this.props.icon == 'string') {
      icons.default.icon = this.props.icon;
    }

    // init tree
    let node = $(ReactDOM.findDOMNode(this));
    this.node = node.jstree({
      core: {
        data: this.props.data,
        themes: {name: 'proton', responsive: true}
      },
      types: icons,
      plugins: plugins
    });

    // tree node selection state changes
    this.node.on('changed.jstree', (e, data) => {
      if (data.action == 'select_node') {
        let selection = this.node.jstree('get_selected')
          , node = data.node.id;
        this.props.onChange(node, selection);
      }
    });

    // tree loaded
    this.node.on('loaded.jstree', () => {
      this.props.onLoaded();
    });
  }

  /**
   * Reload tree data
   * @param data
   */
  reload(data) {
    this.node.jstree(true).settings.core.data = data;
    this.node.jstree(true).refresh();
  }

  /**
   * Select a specified node
   * @param id
   * @param keep {boolean} Whether to clear the previous selected state
   */
  select(id, keep) {

    // clears the previously selected node
    if (!keep) {
      this.node.jstree('get_selected').forEach(id => {
        this.node.jstree('deselect_node', this.node.jstree('get_node', id));
      });
    }

    // select node
    this.node.jstree('select_node', this.node.jstree('get_node', id));
  }

  /**
   * Open the node and all the parent nodes of the node
   * @param id
   */
  open(id) {
    let current = this.node.jstree("get_node", id);
    this.node.jstree('open_node', current);

    let parent = this.node.jstree("get_parent", current);
    while (parent) {
      this.node.jstree('open_node', parent);
      parent = this.node.jstree("get_parent", parent);
    }
  }

  /**
   * Search the tree node.
   * @param keyword
   */
  search(keyword) {

    if (this.timeout) {
      clearTimeout(this.timeout);
    }

    this.timeout = setTimeout(() => {
      this.node.jstree(true).search(keyword);
    }, 250);
  }

  /**
   * Get the selected node.
   * @returns {*}
   */
  getValue() {
    return this.node.jstree('get_selected');
  }
}

Tree.defaultProps = {
  wholerow: false,
  checkbox: false,
  onChange: () => {
  },
  onLoaded: () => {
  }
};

Tree.init = (id, option) => {
  return ReactDOM.render(
    React.createElement(Tree, option),
    document.getElementById(id)
  );
};
