/**
 * @module net.js
 */

'use strict';

import * as util  from './util'
import CONST      from './constant'

/**
 * GET Method.
 * @param url
 * @param data
 * @param callback
 */
export function get(url, data, callback) {

  if (typeof data === 'function') {
    callback = data;
    data = undefined;
  }

  if (window.location.protocol === 'file:') {
    return client(CONST.METHOD_GET, url, data, callback);
  }

  $.ajax({
    type: CONST.METHOD_GET,
    url: url,
    data: data,
    dataType: CONST.DATA_TYPE,
    success: result => {
      callback(result.error, result.data);
    },
    error: err => {
      console.error('ajax ' + url + ' error');
      callback(err);
    }
  });
}

/**
 * POST Method.
 * @param url
 * @param data
 * @param callback
 */
export function post(url, data, callback) {

  if (typeof data === 'function') {
    callback = data;
    data = {};
  }

  if (window.location.protocol === 'file:') {
    return client(CONST.METHOD_POST, url, data, callback);
  }

  $.ajax({
    url: url,
    type: CONST.METHOD_POST,
    async: true,
    data: JSON.stringify(data),
    dataType: CONST.DATA_TYPE,
    contentType: CONST.CONTENT_TYPE_JSON,
    processData: false,
    headers: {
      'csrf-token': util.csrf()
    },
    success: result => {
      callback(result.error, result.data);
    },
    error: err => {
      console.error('ajax ' + url + ' error');
      callback(err);
    }
  });
}

/**
 * PUT Method.
 * @param url
 * @param data
 * @param callback
 */
export function put(url, data, callback) {

  if (typeof data === 'function') {
    callback = data;
    data = {};
  }

  if (window.location.protocol === 'file:') {
    return client(CONST.METHOD_PUT, url, data, callback);
  }

  $.ajax({
    url: url,
    type: CONST.METHOD_PUT,
    async: true,
    data: JSON.stringify(data),
    dataType: CONST.DATA_TYPE,
    contentType: CONST.CONTENT_TYPE_JSON,
    headers: {
      'csrf-token': util.csrf()
    },
    success: result => {
      callback(result.error, result.data);
    }, error: err => {
      console.error('ajax ' + url + ' error');
      callback(err);
    }
  });
}

/**
 * DELETE Method.
 * @param url
 * @param data
 * @param callback
 */
export function remove(url, data, callback) {

  if (typeof data === 'function') {
    callback = data;
    data = {};
  }

  if (window.location.protocol === 'file:') {
    return client(CONST.METHOD_DELETE, url, data, callback);
  }

  $.ajax({
    url: url,
    type: CONST.METHOD_DELETE,
    async: true,
    data: JSON.stringify(data),
    dataType: CONST.DATA_TYPE,
    contentType: CONST.CONTENT_TYPE_JSON,
    processData: false,
    headers: {
      'csrf-token': util.csrf()
    },
    success: result => {
      callback(result.error, result.data);
    },
    error: err => {
      console.error('ajax ' + url + ' error');
      callback(err);
    }
  });
}

/**
 * POST FormData.
 * @param url
 * @param params - QueryParams
 * @param data - FormData
 * @param callback
 * @param progress
 */
export function postData(url, params, data, callback, progress) {

  if (params && Object.keys(params).length > 0) {
    url = url + '?' + $.param(params);
  }

  $.ajax({
    url: url,
    type: CONST.METHOD_POST,
    async: true,
    data: data,
    dataType: CONST.DATA_TYPE,
    contentType: false,
    processData: false,
    headers: {
      'csrf-token': util.csrf()
    },
    xhr: function () {
      let XHR = $.ajaxSettings.xhr();
      if (XHR.upload) {
        XHR.upload.addEventListener('progress', event => {
          if (progress) {
            progress(parseInt(event.loaded / event.total * 10000) / 100);
          }
        }, false);
      }
      return XHR;
    },
    success: result => {
      callback(result.error, result.data || result);
    },
    error: err => {
      console.error('ajax ' + url + ' error');
      callback(err);
    }
  });
}

/**
 * 客户端请求实现
 * @param method
 * @param url
 * @param data
 * @param callback
 */
function client(method, url, data, callback) {

  let request = window.require('request')
    , jar = request.jar()
    , host = util.load(CONST.STORAGE_SCOPE, 'host') || CONST.LIGHT_HOST
    , sid = util.load(CONST.STORAGE_SCOPE, CONST.COOKIE_SESSION_KEY)
    , lang = util.load(CONST.STORAGE_SCOPE, CONST.COOKIE_LANG_KEY);

  if (sid) {
    jar.setCookie(request.cookie(`${CONST.COOKIE_SESSION_KEY}=${sid}`), host);
    jar.setCookie(request.cookie(`${CONST.COOKIE_LANG_KEY}=${lang}`), host);
  }

  let option = {
    url: host + url,
    method: method,
    headers: {
      'content-type': CONST.CONTENT_TYPE_JSON,
      'User-Agent': 'electron',
      'csrf-token': util.csrf()
    },
    jar: jar
  };

  if (method === CONST.METHOD_GET) {
    option.qs = data;
  } else {
    option.json = data;
  }

  console.log(host, option);
  return request(option, (err, response, body) => {

    // 连接错误（远程服务器地址指定错误）
    if (err && !response) {
      return callback(err);
    }

    // 如果是认证错误，跳转到登陆页面
    if (response.statusCode === 401) {
      return window.location = util.load(CONST.STORAGE_SCOPE, 'home');
    }

    // 其他错误，返给客户端
    if (err || response.statusCode !== 200) {
      return callback(err || response.statusCode);
    }

    saveClientCsrfToken(response);
    saveClientSessionID(jar.getCookies(host));

    // 根据content-type，选择如何处理结果
    const contentType = response.headers['content-type'];
    if (contentType === 'image/png') {
      return callback(null, body);
    }

    body = (method === CONST.METHOD_GET) ? JSON.parse(body) : body;
    callback(body.error, body.data);
  });
}


function saveClientCsrfToken(response) {
  const csrftoken = response.headers[CONST.HEADER_CSRF_KEY];
  if (csrftoken) {
    util.save(CONST.STORAGE_SCOPE, CONST.COOKIE_CSRF_KEY, response.headers[CONST.HEADER_CSRF_KEY]);
  }
}


function saveClientSessionID(cookies) {

  let sid = '';
  cookies.forEach(item => {
    if (item.key === 'light.sid') {
      sid = item.value;
    }
  });

  if (sid) {
    util.save('environment', 'light.sid', sid);
  }
}


/**
 * 客户端下载实现
 * @param url
 * @param dest 保存的文件名称（全路径）
 * @param callback
 */
export function download(url, dest, callback) {

  const request = window.require('request')
    , fs = window.require('fs');

  let jar = request.jar()
    , host = util.load(CONST.STORAGE_SCOPE, 'host') || CONST.LIGHT_HOST
    , sid = util.load(CONST.STORAGE_SCOPE, CONST.COOKIE_SESSION_KEY);

  if (sid) {
    jar.setCookie(request.cookie(`${CONST.COOKIE_SESSION_KEY}=${sid}`), host);
  }

  console.log(host + url);
  request({url: host + url, method: CONST.METHOD_GET, headers: {'User-Agent': 'electron'}, jar: jar})
    .on('end', () => {
      if (callback) {
        callback();
      }
    })
    .on('error', err => {
      if (callback) {
        callback(err);
      }
    })
    .pipe(fs.createWriteStream(dest));
}
