/**
 * @module util.js
 */

'use strict';

import CONST from './constant'


export function i(key) {
  if (!window.i18n) {
    return key;
  }

  return window.i18n[key] || key;
}


/**
 *
 */
export function template() {
  _.templateSettings = {
    interpolate: /\{\{-(.+?)\}\}/gim,
    evaluate: /\<\$(.+?)\$\>/gim,
    escape: /\{\{([^-]+?)\}\}/gim
  };
}

/**
 * Obtain a CSRF token from the body
 * If not found in the body, try to get in the cookie
 * @returns {string}
 */
export function csrf() {

  let body = document.getElementById(CONST.BODY_CSRF_KEY);
  if (body && body.value) {
    return encodeURIComponent(body.value);
  }

  let storage = load(CONST.STORAGE_SCOPE, CONST.COOKIE_CSRF_KEY);
  if (storage) {
    return encodeURIComponent(storage);
  }

  let cook = cookie(CONST.COOKIE_CSRF_KEY);
  if (cook) {
    return encodeURIComponent(cook);
  }

  return '';
}

/**
 * uid
 * @returns {*}
 */
export function uid() {

  const body = document.getElementById(CONST.BODY_UID_KEY);
  if (body && body.value) {
    return body.value;
  }

  const storage = load(CONST.STORAGE_SCOPE, 'uid');
  if (storage) {
    return storage;
  }

  const cook = cookie(CONST.COOKIE_UID_KEY);
  if (cook) {
    return cook;
  }
}

export function code() {

  const body = document.getElementById(CONST.BODY_CODE_KEY);
  if (body && body.value) {
    return body.value;
  }

  const storage = load(CONST.STORAGE_SCOPE, 'code');
  if (storage) {
    return storage;
  }

  const cook = cookie('code');
  if (cook) {
    return cook;
  }
}

export function domain() {

  const body = document.getElementById(CONST.BODY_DOMAIN_KEY);
  if (body && body.value) {
    return body.value;
  }

  const storage = load(CONST.STORAGE_SCOPE, 'domain');
  if (storage) {
    return storage;
  }

  const cook = cookie('domain');
  if (cook) {
    return cook;
  }
}

export function isLight() {
  return domain() === 'light';
}

/**
 * param
 * @param url
 * @param key
 * @param val
 * @returns {string}
 */
export function appendParam(url, key, val) {
  let separator = (url.indexOf('?') === -1) ? '?' : '&';
  return url + separator + key + '=' + val;
}

/**
 * param
 * @param name
 * @param url
 * @returns {*}
 */
export function param(name, url) {
  if (!url) {
    url = window.location.href;
  }
  name = name.replace(/[\[\]]/g, '\\$&');

  let regex = new RegExp('[?&]' + name + '(=([^&#]*)|&|#|$)'), results = regex.exec(url);
  if (!results) {
    return '';
  }

  if (!results[2]) {
    return '';
  }
  return decodeURIComponent(results[2].replace(/\+/g, ' '));
}

export function getParam(name, url) {
  return param(name, url);
}

export function params(url) {
  if (!url) {
    return {};
  }

  const param = url.split('?');
  if (param.length < 2) {
    return {};
  }

  return param[1].split('&').reduce((params, param) => {
    const [key, value] = param.split('=');
    params[key] = value ? decodeURIComponent(value.replace(/\+/g, ' ')) : '';
    return params;
  }, {});
}

/**
 * guid4
 * @returns {string}
 */
export function guid4() {
  return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
}

export function guid8() {
  return (((1 + Math.random()) * 0x10000000) | 0).toString(16);
}

/**
 * cookie
 * @param key
 * @returns {string}
 */
export function cookie(key) {
  if (document.cookie) {
    return decodeURIComponent(
      document.cookie.replace(new RegExp('(?:^|.*;\\s*)'
        + encodeURIComponent(key).replace(/[\-\.\+\*]/g, '\\$&') + '\\s*\\=\\s*((?:[^;](?!;))*[^;]?).*'), '$1')
    );
  }

  return load(CONST.STORAGE_SCOPE, key);
}

/**
 * shiftLang
 * @param lang
 */
export function shiftLang(lang) {
  if (document.cookie) {

    // 浏览器
    document.cookie = CONST.COOKIE_LANG_KEY + '=' + encodeURIComponent(lang) + ';path=/';
  } else {

    // 客户端
    save(CONST.STORAGE_SCOPE, CONST.COOKIE_LANG_KEY, lang);
  }
}

/**
 * save
 * @param scope
 * @param key
 * @param val
 */
export function save(scope, key, val) {
  if (!window.localStorage) {
    return;
  }

  let storage = window.localStorage.getItem(scope);
  storage = storage ? JSON.parse(storage) : {};

  storage[key] = val;
  window.localStorage.setItem(scope, JSON.stringify(storage));
}

export function clear(scope, key) {
  if (!window.localStorage) {
    return;
  }

  let storage = window.localStorage.getItem(scope);
  storage = storage ? JSON.parse(storage) : {};

  delete storage[key];
  window.localStorage.setItem(scope, JSON.stringify(storage));
}

/**
 * load
 * @param scope
 * @param key
 * @returns {Object}
 */
export function load(scope, key) {
  if (!window.localStorage) {
    return undefined;
  }

  let storage = window.localStorage.getItem(scope);
  if (storage) {
    return JSON.parse(storage)[key];
  }

  return undefined;
}

export function htmlEncode(value) {
  return $('<div/>').text(value).html();
}

export function htmlDecode(value) {
  return $('<div/>').html(value).text();
}

export function clone(val) {
  return require('./clone')(val);
}
