/**
 * @file
 * @module form-fieldset.js
 * @author r2space@gmail.com
 * @version 1.0.0
 */

'use strict';

import React        from 'react'
import ReactDOM     from 'react-dom'
import CONST        from '../constant'
import Widget       from '../widget/widget'
import i            from './localize'


export default class FieldSet extends React.Component {

  constructor(props) {
    super(props);
    this.state = {};
  }

  render() {

    if (this.props.option.mode === CONST.MODE.INPUT) {
      if (this.props.data.length <= 0) {
        return this.empty();
      }
    }

    // 获取行数, 通过查看数据中最大的row值来获取总行数
    const max = _.maxBy(this.props.data, item => item.row);
    let totalRow = max ? max.row + 1 : 0;

    // 设计模式时, 最小行数为7
    if (this.props.option.mode === CONST.MODE.DESIGN) {
      totalRow = Math.max(totalRow || 0, this.props.option.h);
    }

    return dom('fieldset', {style: this.style()},
      _.range(totalRow).map((row, index) => {

        let data = this.props.data.filter(item => item.row === row);

        // 如果是空行，则跳过显示
        if (this.isEmptyRow(data)) {
          return null;
        }

        // 获取行数据, 并以col字段排序显示
        return React.createElement(Row, {
          key: `row-${index}`,
          row: index,
          data: data,
          option: this.props.option,
          emit: this.props.emit,
          style: this.props.style
        });
      })
    );
  }

  empty() {
    return dom('fieldset', {style: {padding: '30px', textAlign: 'center'}}, 'NO DATA');
  }

  isEmptyRow(row) {

    if (this.props.option.mode === CONST.MODE.DESIGN) {
      return false;
    }

    return (row.length <= 0);
  }

  style() {

    let style = {display: this.props.option.closed ? 'none' : 'block'};

    if (this.props.option.scroll) {
      style.maxHeight = this.state.height;
      style.overflowX = 'hidden';
      style.overflowY = 'auto';
    }
    return style;
  }

  componentDidMount() {

    if (!this.props.option.scroll) {
      return;
    }

    // 调整浏览器窗口大小时，重新计算form大小，让form在一页内显示。通常在固定form位置时使用，如属性画面
    let node = ReactDOM.findDOMNode(this);
    this.setState({height: (window.innerHeight - node.offsetTop - 240) + 'px'});

    $(window).resize(() => {
      this.setState({height: (window.innerHeight - node.offsetTop - 240) + 'px'});
    });
  }
}

FieldSet.defaultProps = {
  data: [],
  option: {},
  style: {}
};


/**
 * 表单的一行
 */
class Row extends React.Component {
  constructor(props) {
    super(props);
  }

  render() {

    // 一个组件可以通过parents, 查看所有兄弟组件的状态, 来控制是否可以扩展, 是否可以移动等
    this.props.option.parents = this.props.data;

    let items = [], col = 0;
    for (let index = 0; index < this.props.option.w; index++) {

      let item = this.props.data.find(item => item.col === col);
      item = item || {row: this.props.row, col: col, rowSpan: 1, colSpan: 1};

      items.push(React.createElement(Section, {
        key: `col-${this.props.row}-${col}`,
        data: item,
        option: this.props.option,
        emit: this.props.emit,
        style: this.props.style
      }));

      col++;
      if (item.colSpan > 1) {
        index = index + item.colSpan - 1;
      }
    }

    // 隐藏被折叠的行
    let hide = _.indexOf(this.props.option.hidden, this.props.row) >= 0;
    return dom('div', {className: 'row', style: {display: hide ? 'none' : 'block'}}, items);
  }
}

Row.defaultProps = {
  data: [],
  option: {},
  style: {}
};


/**
 * 放置组件的单元格, 允许拖拽添加, 拖拽移动, 允许横向扩展
 */
class Section extends React.Component {
  constructor(props) {
    super(props);

    this.state = {hover: false};
    this.onClick = this.onClick.bind(this);
    this.onRemoveClick = this.onRemoveClick.bind(this);
    this.onAddRowClick = this.onAddRowClick.bind(this);
    this.onCopyClick = this.onCopyClick.bind(this);
    this.onRemoveRowClick = this.onRemoveRowClick.bind(this);
    this.onExpand = this.onExpand.bind(this);
  }

  render() {

    const style = _.extend({
      backgroundColor: this.backgroundColor()
    }, this.props.style.section);

    const required = this.props.data.validator === 'required';

    return dom('section', {
        className: this.className(),
        style: style,
        onClick: this.onClick,
        onMouseEnter: () => this.setState({hover: true}),
        onMouseLeave: () => this.setState({hover: false})
      },
      dom('label', {className: 'label'},
        this.props.data.title,
        required ? dom('span', {className: 'color-red'}, ' ＊') : null
      ),
      this.widget(),
      this.expander(),
      this.action()
    );
  }

  widget() {
    let dom = Widget.defaultProps.control[this.props.data.type];

    if (dom) {
      return React.createElement(dom, this.props);
    }
    return null;
  }

  action() {

    if (this.props.option.mode !== CONST.MODE.DESIGN) {
      return;
    }

    if (this.props.actionDisplayMode === 'select') {
      if (!this.isSelected()) {
        return null;
      }
    }

    if (this.props.actionDisplayMode === 'hover') {
      if (!this.state.hover) {
        return null;
      }
    }

    return dom('ul', {className: 'action-button-right nav nav-pills'},
      dom('li', {},
        dom('a', {className: 'dropdown-toggle', 'data-toggle': 'dropdown'},
          dom('i', {className: 'fa fa-fw fa-pencil-square-o'})
        ),
        dom('ul', {className: 'dropdown-menu dropdown-menu-right'},
          dom('li', {onClick: this.onAddRowClick},
            dom('a', {href: '#'}, dom('i', {className: 'fa fa-fw fa-plus-circle'}), ' ' + i('body.form.add.row'))
          ),
          this.props.data.type ?
            dom('li', {onClick: this.onCopyClick},
              dom('a', {href: '#'}, dom('i', {className: 'fa fa-fw fa-copy'}), ' ' + i('body.form.copy.widget'))
            )
            : null,
          dom('li', {className: 'divider'}),
          this.props.data.type && this.props.option.w > 1
            ? dom('li', {onClick: this.onRemoveClick},
              dom('a', {href: '#'}, dom('i', {className: 'fa fa-fw fa-trash'}), ' ' + i('body.form.remove.widget'))
            )
            : null,
          dom('li', {onClick: this.onRemoveRowClick},
            dom('a', {href: '#'}, dom('i', {className: 'fa fa-fw fa-trash'}), ' ' + i('body.form.remove.row'))
          )
        )
      )
    );
  }

  onAddRowClick(event) {

    event.preventDefault();
    event.stopPropagation();

    this.props.emit(CONST.EVENT.ADDROW, this.props.data);
  }

  onCopyClick(event) {

    event.preventDefault();
    event.stopPropagation();

    this.props.emit(CONST.EVENT.COPY, this.props.data);
  }

  onRemoveClick(event) {
    event.preventDefault();
    event.stopPropagation();

    this.props.emit(CONST.EVENT.REMOVE, this.props.data);
  }

  onRemoveRowClick(event) {
    event.preventDefault();
    event.stopPropagation();
    this.props.emit(CONST.EVENT.REMOVEROW, this.props.data);
  }

  className() {

    let name = this.paddingLeft();

    name = name + ' col-sm-' + (12 / this.props.option.w * this.props.data.colSpan);

    if (this.props.option.mode === CONST.MODE.DESIGN) {
      name = name + ' widget';
    }

    return name;
  }

  /**
   * 根据组件所处的层级，进行缩进
   * @returns {*}
   */
  paddingLeft() {
    if (this.props.data.col > 0) {
      return '';
    }

    if (this.props.data.path) {
      return `padding-left-${this.props.data.path.length}`;
    }
    return '';
  }

  backgroundColor() {
    if (this.props.option.mode !== CONST.MODE.DESIGN) {
      return null;
    }

    return this.isSelected() ? this.props.bgColorSelected : this.props.bgColor;
  }

  isSelected() {
    if (!this.props.option.selected) {
      return false;
    }

    let isRow = this.props.data.row === this.props.option.selected[0]
      , isCol = this.props.data.col === this.props.option.selected[1];

    return isRow && isCol;
  }

  /**
   * 横向扩展组件, 光标移动到边缘时有右向剪头出现
   * @returns {DOM}
   */
  expander() {

    if (this.props.option.mode !== CONST.MODE.DESIGN) {
      return null;
    }

    if (!this.props.data.type) {
      return null;
    }

    if ((this.props.data.col + this.props.data.colSpan) >= this.props.option.w) {
      return null;
    }

    let hasSibling = _.find(this.props.option.parents, item => {
      return item.col === this.props.data.col + 1;
    });

    if (hasSibling) {
      return null;
    }

    return dom('div', {className: 'ui-resizable-handle ui-resizable-e', onClick: this.onExpand});
  }

  onExpand() {
    this.props.emit(CONST.EVENT.EXPAND, {
      col: this.props.data.col, colSpan: this.props.data.colSpan, row: this.props.data.row
    }, ReactDOM.findDOMNode(this));
  }

  onClick() {
    if (this.props.option.mode !== CONST.MODE.DESIGN) {
      return;
    }

    this.props.emit(CONST.EVENT.SELECT, this.props.data, ReactDOM.findDOMNode(this));
  }

  componentDidMount() {
    this.droppable();
    this.draggable();
  }

  componentDidUpdate() {
    this.droppable();
    this.draggable();
  }

  droppable() {
    if (this.props.option.mode !== CONST.MODE.DESIGN) {
      return;
    }

    let node = $(ReactDOM.findDOMNode(this));
    if (!node.droppable('instance')) {
      node.droppable({
        accept: this.props.accept,
        activeClass: 'widget-active',
        hoverClass: 'widget-hover',
        drop: (event, ui) => {

          let type = $(ui.draggable).attr('name')
            , title = $(ui.draggable).attr('data-title');

          // 添加组件
          if (type) {
            return this.props.emit(CONST.EVENT.ADD,
              {col: this.props.data.col, row: this.props.data.row, type: type, title: title},
              ReactDOM.findDOMNode(this));
          }

          // 移动组件
          return this.props.emit(CONST.EVENT.MOVE, {
            col: this.props.data.col,
            row: this.props.data.row,
            rowSpan: this.props.data.rowSpan,
            colSpan: this.props.data.colSpan,
            source: Section.temporal
          }, ReactDOM.findDOMNode(this));
        }
      });
    }

    node.droppable(this.props.data.type ? 'disable' : 'enable');
  }

  draggable() {
    if (this.props.option.mode !== CONST.MODE.DESIGN) {
      return;
    }

    let node = $(ReactDOM.findDOMNode(this));

    if (!node.draggable('instance')) {
      node.draggable({
        revert: 'invalid', opacity: 0.7,
        helper: () => {

          // 设定宽度与Zindex，放在拖动时组件变形
          let item = node.clone().css({'z-index': '10', width: node.innerWidth()});

          // 删除右边的编辑按钮
          item.find('.action-button-right').remove();

          return item;
        },
        start: () => {
          Section.temporal = {
            col: this.props.data.col,
            row: this.props.data.row,
            colSpan: this.props.data.colSpan,
            rowSpan: this.props.data.rowSpan
          };
        }
      });
    }

    node.draggable(this.props.data.type ? 'enable' : 'disable');
  }

}

Section.defaultProps = {
  actionDisplayMode: 'select',         // 右上角操作按钮显示模式 取值：select, hover
  data: [],
  option: {},
  style: {},
  accept: '.control button, section', // 接受拖拽的起点容器（允许从控件列表拖拽添加组件，允许拖拽移动组件）
  bgColor: '#FAFCFD',
  bgColorSelected: '#F5F5F5'
};