/**
 * @module properties.js
 */

'use strict';

import React    from 'react'
import ReactDOM from 'react-dom'
import CONST    from '../constant'
import Header   from './form-header'
import FieldSet from './form-fieldset'
import Plugin   from './plugin'
import Footer   from './form-footer'
import Control  from './control'
import i        from './localize'


export default class Properties extends React.Component {

  constructor(props) {
    super(props);

    this.emitter = this.props.emitter || $({});

    this.emit = this.emit.bind(this);
    this.onChange = this.onChange.bind(this);
    this.onFold = this.onFold.bind(this);
    this.onPlugin = this.onPlugin.bind(this);
    this.onWidgetSelected = this.onWidgetSelected.bind(this);
    this.onFormSelected = this.onFormSelected.bind(this);

    this.state = {
      type: CONST.PROPS_TYPE_FORM,                  // 属性框类型
      data: this.formatProperties(this.props.data), // 控件 & 数据
      option: {
        plugin: this.props.plugin,                  // 是否显示插件
        mode: CONST.MODE.INPUT,                     // 输入模式
        closed: false,                              // 是否是关闭状态
        scroll: true,                               // 固定高度，超过屏幕时显示滚动条
        w: 4,                                       // 列数
        form: this.props.form,
        name: ''                                    // 当前操作的组件name
      }
    };

    // 属性值发生变化，通知表单画面
    this.emitter.on(CONST.EVENT.CHANGE, this.onChange);

    // 折叠属性框
    this.emitter.on(CONST.EVENT.FOLDFORM, this.onFold);

    // 显示插件管理器
    this.emitter.on(CONST.EVENT.PLUGIN, this.onPlugin);

    // 选择组件时，显示组件属性
    this.props.form.emitter.on(CONST.EVENT.SELECT, this.onWidgetSelected);
    this.props.form.emitter.on(CONST.EVENT.SELECTFORM, this.onFormSelected);
  }


  render() {

    if (this.original) {
      this.state.option.name = this.original.name;
    }

    return dom('form', {className: 'light-form'},
      React.createElement(Header, {
        title: this.title(),
        closed: this.state.option.closed,
        plugin: this.state.option.plugin,
        emit: this.emit
      }),
      this.state.option.plugin
        ? React.createElement(Plugin, {type: this.state.type, data: this.original, emit: this.emit})
        : React.createElement(FieldSet, {data: this.state.data, option: this.state.option, emit: this.emit}),
      React.createElement(Footer, {button: this.props.button})
    );
  }


  componentDidMount() {
    this.setFormDefaults();
  }


  setFormDefaults() {

    // 初始化表单的缺省值
    _.flatten(this.props.propertie[CONST.PROPS_TYPE_FORM]).forEach(item => {
      if (_.isUndefined(item.defaults)) {
        return;
      }
      if (_.isUndefined(this.props.form.meta[item.name])) {
        this.props.form.meta[item.name] = item.defaults;
      }
    });
  }


  title() {
    if (this.state.type === CONST.PROPS_TYPE_FORM) {
      return i('body.form.property');
    }

    const control = Control.defaultProps.control.find(item => item.name === this.state.type);
    return control ? `${i(control.title || control.description)} ${i('body.form.property')}` : i('body.form.property');
  }


  /**
   * 发射事件（当前属性表单内有效，防止一个页面有多个Form时，事件互相影响）
   */
  emit(type, data, node) {
    this.emitter.trigger(type, [data, node]);
  }


  /**
   * 属性头的折叠事件
   */
  onFold(event, state) {
    this.state.option.closed = state;
    this.setState({option: this.state.option});
  }


  /**
   * 显示插件一览
   */
  onPlugin(event, state) {
    this.state.option.plugin = state;
    this.setState({option: this.state.option});
  }


  /**
   * 属性值发生了变化
   * @param event
   * @param data
   */
  onChange(event, data) {

    if (this.state.type === CONST.PROPS_TYPE_FORM) {

      this.original[data.name] = data.value;
      this.props.form.setFormInfo(data, {[data.name]: data.value});

      this.props.onChange(data, this.state.data, CONST.PROPS_TYPE_FORM);
      return this.onFormSelected();
    }

    this.original[data.name] = data.value;
    this.props.form.setWidget(data, this.original);

    this.props.onChange(data, this.state.data, this.state.type);
    this.onWidgetSelected(null, this.original);
  }


  /**
   * 在From面板，选择了某个组件
   * @param event
   * @param data
   * @returns {*}
   */
  onWidgetSelected(event, data) {

    if (data.type) {
      this.original = data;
      this.state.data = this.formatProperties(this.props.propertie[data.type]);

      this.state.data = this.props.onWidgetWillShow(this.state.data, CONST.PROPS_TYPE_WIDGET);
      return this.setState({type: data.type, data: this.state.data});
    }

    // 选中的是空栏时
    this.original = [];
    this.original = this.props.onWidgetWillShow(this.original, CONST.PROPS_TYPE_WIDGET);
    this.setState({type: data.type, data: this.original});
  }


  onFormSelected() {

    this.original = this.props.form.meta;

    let props = this.props.propertie[CONST.PROPS_TYPE_FORM];
    props = this.props.onWidgetWillShow(props, CONST.PROPS_TYPE_FORM);

    this.state.data = this.formatProperties(props);
    this.setState({type: CONST.PROPS_TYPE_FORM, data: this.state.data});
  }


  formatProperties(data = []) {

    let result = [];

    data.forEach((item, row) => {

      if (!Array.isArray(item)) {
        item = [item];
      }

      item.forEach((sub, col) => {

        // 多国语言对应
        Properties.translate(sub);

        // 配置文件里，为了方便没有填写组件所在的行数及宽带，这里根据顺序计算row, col等信息
        sub.row = row;
        sub.col = col;
        sub.rowSpan = 1;
        sub.colSpan = 4 / item.length;

        // 设定取值 (或缺省值)
        sub.value = _.isUndefined(this.original[sub.name]) ? sub.defaults : this.original[sub.name];
        result.push(sub);
      });
    });

    return result;
  }


  static translate(item) {

    // 遍历所有项目，字符串类型的值，尝试使用多国语言进行转换
    _.each(item, (itemVal, itemKey) => {

      if (_.isString(itemVal)) {
        item[itemKey] = i(itemVal);
        return;
      }


      if (_.isArray(itemVal)) {
        item[itemKey] = _.map(itemVal, val => {
          if (_.isString(val)) {
            return i(val);
          }

          if (_.isArray(val)) {
            return _.map(val, v => i(v));
          }

          if (_.isObject(val)) {
            return _.each(val, (v, k) => val[k] = i(v));
          }

          return itemVal;
        });
        return;
      }

      if (_.isObject(itemVal)) {
        item[itemKey] = _.each(itemVal, (val, key) => {
          itemVal[key] = i(val);
        });
        return;
      }

      item[itemKey] = itemVal;
    });
  }

}


Properties.defaultProps = {
  emitter: undefined,
  propertie: {},
  form: {},
  data: [],
  button: [],     // 操作按钮
  onChange: (source, widgets, type) => {
  },
  onWidgetWillShow: (widgets, type) => {
    return widgets;
  }
};


Properties.init = (node, option) => {
  return ReactDOM.render(
    React.createElement(Properties, option),
    document.getElementById(node)
  );
};
