/**
 * @file
 * @module widget-tag.js
 * @version 1.0.0
 */

'use strict';

import Widget   from './widget'
import CONST    from '../constant'

export default class Tag extends Widget {

  constructor(props) {
    super(props);
    this.state = {size: 1, editor: -1};

    this.onRemove = this.onRemove.bind(this);
    this.onKeyDown = this.onKeyDown.bind(this);
    this.onChange = this.onChange.bind(this);
  }

  componentDidUpdate() {
    if (this.state.editor > -1) {
      this.current.children('input').focus();
    }
  }

  render() {
    return dom('div', {className: 'bootstrap-tagsinput', onClick: this.onFocus}, this.tags(), this.widget());
  }

  /**
   * 获取输入框Component
   * size为隐藏的输入框宽度，当值为空时，为了显示placeholder内容，调整size为placeholder长度
   * @returns {*}
   */
  widget() {
    if (this.disabled()) {
      return null;
    }

    return dom('input', {
      style: {
        border: 'none',
        outline: 'none',
        padding: '1px',
        margin: 0,
        width: 'auto !important',
        backgroundColor: 'transparent',
        boxShadow: 'none',
        maxWidth: 'inherit',
        paddingLeft: '10px'
      },
      type: 'text',
      size: this.getValue().length <= 0 ? this.count(this.props.data.placeholder) : this.state.size,
      placeholder: this.getValue().length <= 0 ? this.props.data.placeholder : '',
      onKeyDown: this.onKeyDown,
      onChange: this.onChange
    });
  }

  getValue() {
    if (!this.value()) {
      return [];
    }
    if (_.isString(this.value())) {
      return this.value().split(',');
    }
    return this.value();
  }

  /**
   * 生成Tag的Component
   * @returns {*}
   */
  tags() {
    if (this.getValue() <= 0) {
      return null;
    }

    return this.getValue().map((value, index) => {

      let remove = null, style = null, text = null;

      // 只读状态下，去除删除按钮的宽度
      if (this.disabled()) {
        style = {paddingRight: '8px'};
      } else {
        remove = dom('span', {'data-role': 'remove', 'data-index': index, onClick: this.onRemove});
      }

      // 编辑模式时, 显示输入框
      if (!this.disabled() && index == this.state.editor) {
        text = dom('input', {
          style: {backgroundColor: '#18568C'}, value: value,
          onChange: this.onEditorChange, onBlur: this.onEditorDone
        });
      } else {
        text = value;
      }

      return dom('span', {
        key: text + index, className: 'tag label label-info', style: style, onClick: this.onEdit, 'data-index': index
      }, text, remove);
    });
  }

  /**
   * 点击tag的某个项目, 进入编辑模式
   * @param event
   */
  onEdit(event) {
    this.current = $(event.target).parent();
    this.setState({editor: Number($(event.target).parent().attr('data-index'))});
  }

  /**
   * 编辑模式下, 修改文本框的内容
   * @param event
   */
  onEditorChange(event) {

    let index = Number($(event.target).parent().attr('data-index'))
      , result = this.getValue();

    result[index] = $(event.target).val();
    this.change(_.compact(result), this.props.data.name);
  }

  /**
   * 编辑模式结束
   */
  onEditorDone() {
    this.setState({editor: -1});
  }

  onFocus(event) {
    $(event.target).children('input').focus();
  }

  /**
   * 输入框里输入值时，为了显示输入的所有文字，调整输入框的宽度
   * @param event
   */
  onChange(event) {
    let size = this.count(event.target.value);
    this.setState({size: size < 1 ? 1 : size});
  }

  onKeyDown(event) {

    let result = this.getValue();

    if (event.keyCode == CONST.KEY.BACKSPACE) {
      if (event.target.value.length == 0) {
        result.splice(result.length - 1, 1);
      }
    }

    if (event.keyCode == CONST.KEY.DELETE) {
      result.splice(result.length - 1, 1);
    }

    if (event.keyCode == CONST.KEY.ENTER) {
      event.preventDefault();

      if (event.target.value.length > 0) {

        // 如果输入有 [:] 符号，:号前面的文字不能包含 [.]
        let match = event.target.value.match(/^(.*)[ ]*:[ ]*(.*)$/);
        if (match) {
          match[1] = match[1].replace(/\./g, '');
          match = (match[1] + ':' + match[2]).replace(/^:*|:*$/g, '');
        }

        result.push(match || event.target.value);
        $(event.target).val('');
        this.setState({size: 1});
      }
    }

    this.change(_.compact(result), this.props.data.name);
  }

  /**
   * 删除按钮
   * @param event
   */
  onRemove(event) {
    let result = this.getValue();
    result.splice($(event.target).attr('data-index'), 1);
    this.change(_.compact(result), this.props.data.name);
  }
}
