/**
 * @file 表单组件父类
 * @module widget.js
 * @author r2space@gmail.com
 * @version 1.0.0
 */

'use strict';

import React    from 'react'
import ReactDOM from 'react-dom'
import CONST    from '../constant'


export default class Widget extends React.Component {

  constructor(props) {
    super(props);
  }

  icon() {
    if (!this.props.data.textIcon) {
      return null;
    }

    // 图标显示的位置 append: 在前面显示 prepend: 在后端显示
    let append = 'icon-' + (this.props.data.textIconAppend === 'append' ? 'append' : 'prepend')
      , icon = 'fa-' + this.props.data.textIcon;

    return dom('i', {className: append + ' fa ' + icon});
  }

  value() {
    return _.isUndefined(this.props.data.value)
      ? this.props.data.defaults
      : this.props.data.value;
  }

  disabled() {
    if (this.props.option.mode === CONST.MODE.VIEW) {
      return 'disabled';
    }

    if (this.props.option.mode === CONST.MODE.READONLY) {
      return 'disabled';
    }

    return this.boolean(this.props.data.disabled) ? 'disabled' : '';
  }

  change(value, name) {
    this.props.emit(CONST.EVENT.CHANGE, {
      value: value,
      name: name,
      col: this.props.data.col,
      row: this.props.data.row
    }, ReactDOM.findDOMNode(this));
  }

  /**
   * 获取文字数，一个汉字计数2，英文字母计数1
   * @param str
   * @returns {number}
   */
  count(str) {
    if (!str) {
      return 0;
    }

    let len = 0;
    for (let i = 0; i < str.length; i++) {
      if (str.charCodeAt(i) > 255) len++;
      len++;
    }
    return len;
  }

  /**
   * 字符串转换为Boolean
   * @param string
   * @returns {*}
   */
  boolean(string) {
    if (_.isNull(string) || _.isUndefined(string)) {
      return false;
    }

    if (_.isBoolean(string)) {
      return string;
    }

    switch (string.toLowerCase().trim()) {
      case 'true':
      case 'yes':
      case 'on':
      case '1':
        return true;
      case 'false':
      case 'no':
      case 'off':
      case '0':
      case null:
        return false;
      default:
        return true;
    }
  }

  number(string) {
    let result = Number(string);
    return _.isNaN(result) ? 0 : result;
  }

  normalize(option) {
    return Widget.normalize(option);
  }

  array(val) {
    if (!val) {
      return [];
    }

    return Array.isArray(val) ? val : [val];
  }

  /**
   * 上传文件
   * TODO: 支持CDN
   * @param event
   * @returns {boolean}
   */
  upload(event) {
    let files = event.target.files;
    if (!files || files.length <= 0) {
      return false;
    }

    let formData = new FormData();
    for (let i = 0; i < files.length; i++) {
      formData.append('files', files[i]);
    }

    light.multipart(
      CONST.Config.FileUploadAPI, {data: this.props.data.fileOption}, formData, this.onFinish, this.onProcess
    );
  }

};

Widget.defaultProps = {
  control: {}
};

Widget.normalize = function (option) {
  if (!option) {
    return option;
  }

  if (_.isObject(option)) {
    return option;
  }

  let match = option.match(/^(.*)[ ]*:[ ]*(.*)$/);
  if (match) {
    return {value: match[1], title: match[2]};
  }

  return {value: option, title: option};
};

Widget.guid = () => {
  return light.util.guid8();
};

Widget.clone = (val) => {
  return light.util.clone(val);
};


Widget.emitter = $({});

/**
 * 渲染组件
 * @param container
 * @param setting
 * @param mode 模式，可以取值（INPUT | READONLY | VIEW）
 * @param form 整个表单数据（组件之间引用数据时参照）
 * @returns {*}
 */
Widget.init = (container, setting, mode, form) => {

  const node = typeof container === 'string' ? document.getElementById(container) : container;
  setAttribute(node, 'data-type', setting.type.toLowerCase());
  setAttribute(node, 'data-name', setting.name);
  setAttribute(node, 'data-value', setting.value);

  return ReactDOM.render(React.createElement(
    AloneWidget, {setting: setting, mode: mode, form: form, emitter: Widget.emitter}
  ), node);
};

/**
 * 获取组件的取值
 * @param reactElement
 */
Widget.get = (reactElement) => {
  return reactElement.state.setting.value;
};

/**
 * 设定组件的取值
 * @param reactElement
 * @param value
 */
Widget.set = (reactElement, value) => {
  setAttribute(ReactDOM.findDOMNode(reactElement).parentElement, 'data-value', value);

  reactElement.state.setting.value = value;
  reactElement.setState({data: reactElement.state.setting});
};

function setAttribute(node, key, value) {

  // 数组
  if (Array.isArray(value)) {
    return node.setAttribute(key, JSON.stringify(value));
  }

  // 哈希
  if (isHash(value)) {
    return node.setAttribute(key, JSON.stringify(value));
  }

  node.setAttribute(key, value);
}

function isHash(val) {
  return (!!val) && (val.constructor === Object);
}

/**
 * 单个组件渲染用的组件
 */
class AloneWidget extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      setting: this.props.setting
    };
    this.control = Widget.defaultProps.control[this.props.setting.type];
  }

  render() {

    const required = this.state.setting.validator === 'required';

    return dom('div', {},
      dom('label', {className: 'label'},
        this.state.setting.title,
        required ? dom('span', {className: 'color-red'}, ' ＊') : null,
      ),
      React.createElement(this.control, {
          data: this.state.setting,
          option: {mode: this.props.mode || CONST.MODE.INPUT, form: this.props.form, emitter: this.props.emitter},
          emit: (event, data) => {

            // 更新参数
            this.state.setting.value = data.value;
            this.setState({data: this.state.setting});

            // 发送变更通知
            Widget.emitter.trigger(CONST.EVENT.CHANGE, [data, this.state.setting.name]);

            // 添加到属性
            setAttribute(ReactDOM.findDOMNode(this).parentElement, 'data-value', data.value);
          }
        }
      )
    );
  }
}