/**
 * @file 工作流
 * @module workflow
 * @author r2space@gmail.com
 * @version 1.0.0
 */

'use strict';

import Activity from './workflow-activity'
import Header   from './workflow-header'
import Footer   from './workflow-footer'
import CONST    from '../constant'


export default class Process extends React.Component {

  constructor(props) {
    super(props);

    this.emitter = this.props.emitter || $({});
    this.emit = this.emit.bind(this);
    this.activities = Activity.toTree(this.props.activities);

    this.state = {
      emit: this.emit,
      meta: this.props.meta,
      mode: this.props.mode,
      style: this.props.style,

      activities: this.activities,
      selected: {}
    };

    this.onAddActivity = this.onAddActivity.bind(this);
    this.onAddSubActivity = this.onAddSubActivity.bind(this);
    this.onAddCondition = this.onAddCondition.bind(this);
    this.onRemoveCondition = this.onRemoveCondition.bind(this);
    this.onRemoveActivity = this.onRemoveActivity.bind(this);
    this.onSelectActivity = this.onSelectActivity.bind(this);
    this.onSelectCondition = this.onSelectCondition.bind(this);

    this.emitter.on(CONST.EVENT.ADDACTIVITY, this.onAddActivity);
    this.emitter.on(CONST.EVENT.ADDSUBACTIVITY, this.onAddSubActivity);
    this.emitter.on(CONST.EVENT.ADDCONDITION, this.onAddCondition);
    this.emitter.on(CONST.EVENT.REMOVECONDITION, this.onRemoveCondition);
    this.emitter.on(CONST.EVENT.REMOVEACTIVITY, this.onRemoveActivity);

    this.emitter.on(CONST.EVENT.SELECTACTIVITY, this.onSelectActivity);
    this.emitter.on(CONST.EVENT.SELECTCONDITION, this.onSelectCondition);
  }

  render() {
    return dom('div', null,
      dom(Header, this.state),
      dom(Activity, this.state),
      dom(Footer, Object.assign({
        button: this.props.footerButton, description: this.props.footerDescription
      }, this.state))
    );
  }

  emit(type, data, node) {
    this.emitter.trigger(type, [data, node]);
  }

  onSelectActivity(event, activity, index) {
    this.setState({selected: {id: index.aid}});
  }

  onSelectCondition(event, activity, index) {
    this.setState({selected: {id: index.aid, transition: index.tid, condition: index.cid}});
  }

  onAddCondition(event, isAndCondition) {

    const index = this.state.selected;

    // 获取 condition 非空的 transition
    const transitions = this.props.activities.find(item => item.id === index.id).transitions
      , transition = transitions.find(item => item.id === index.transition);

    if (isAndCondition && transition.condition[index.condition]) {
      // 添加AND条件
      transition.condition[index.condition].push({key: '', operator: 'gt', value: ''});
    } else {
      // 添加OR条件
      transition.condition.push([{key: '', operator: 'gt', value: ''}]);
    }

    this.activities = Activity.toTree(this.props.activities);
    this.setState({activities: this.activities});

    // 通知属性窗口
    this.emit(CONST.EVENT.SELECTCONDITION, transition.condition[index.condition], {
      aid: index.id,
      tid: index.transition,
      cid: index.condition
    });
  }

  onRemoveCondition(event, isAndCondition) {

    const index = this.state.selected;

    // 获取 condition 非空的 transition
    const transitions = this.props.activities.find(item => item.id === index.id).transitions
      , transition = transitions.find(item => item.id === index.transition);

    let cid = index.condition;

    if (isAndCondition) {
      // 删除AND条件
      transition.condition[index.condition].pop();
      transition.condition = transition.condition.filter(item => item.length > 0);
    } else {
      // 删除OR条件
      transition.condition.splice(index.condition, 1);
      cid = index.condition - 1;
    }

    cid = cid < 0 ? 0 : cid;
    cid = cid >= transition.condition.length ? transition.condition.length - 1 : cid;

    this.activities = Activity.toTree(this.props.activities);
    this.setState({activities: this.activities});

    // 通知属性窗口
    this.emit(CONST.EVENT.SELECTCONDITION, transition.condition[cid], {
      aid: index.id,
      tid: index.transition,
      cid: cid
    });
  }

  onRemoveActivity(event) {

    const id = this.state.selected.id
      , activity = this.props.activities.find(item => item.id === id)
      , transition = activity.transitions.find(item => !item.condition);

    // 拷贝当前 activity 的关联关系到前序 activity
    const prev = this.props.activities.find(activity => activity.transitions.find(transition => transition.to === id));
    prev.transitions = prev.transitions.concat(activity.transitions.filter(item => item.condition));

    // 如果有后续 activity，将这个后续的 activity 设置到前序 activity
    if (transition.to) {
      prev.transitions.forEach(item => {
        item.to = item.to === id ? transition.to : item.to;
      });
    }

    // 如果后续没有 activity，就删除关联关系
    else {
      prev.transitions = prev.transitions.filter(item => !(item.condition && item.to === id));
    }

    // 删除选中的 activity
    this.props.activities = this.props.activities.filter(item => item.id !== id);

    this.activities = Activity.toTree(this.props.activities);
    this.setState({activities: this.activities});

    this.emit(CONST.EVENT.SELECTACTIVITY, prev, {aid: prev.id});
  }

  onAddActivity(event, id) {

    // 获取 condition 为空的 transition
    const transitions = this.props.activities.find(item => item.id === id).transitions
      , transition = transitions.find(item => item.condition === undefined);

    // 添加新的 activity
    const activity = {
      id: light.util.guid8(),
      transitions: [{id: light.util.guid8(), to: transition.to}]
    };
    this.props.activities.push(activity);

    // 修改原始的 activity 连接信息（上游 activity）
    transition.to = activity.id;

    this.activities = Activity.toTree(this.props.activities);
    this.setState({activities: this.activities});

    // 选中
    this.emit(CONST.EVENT.SELECTACTIVITY, activity, {aid: activity.id});
  }

  onAddSubActivity(event, id) {

    const transitions = this.props.activities.find(item => item.id === id).transitions;

    // 添加新的 activity
    const activity = {
      id: light.util.guid8(),
      transitions: [{id: light.util.guid8(), to: ''}]
    };
    this.props.activities.push(activity);

    // 设置 activity 连接信息
    transitions.push({
      id: light.util.guid8(),
      to: activity.id,
      condition: [[{key: '', operator: 'eq', value: ''}]]
    });

    this.activities = Activity.toTree(this.props.activities);
    this.setState({activities: this.activities});

    // 选中
    this.emit(CONST.EVENT.SELECTACTIVITY, activity, {aid: activity.id});
  }

  setBranchInfo(index, data) {

    // Use the index information to find the condition
    const activity = this.props.activities.find(item => item.id === index.aid)
      , transition = activity.transitions.find(item => item.id === index.tid)
      , condition = transition.condition[index.cid];

    const key = Object.keys(data)[0];
    const [name, i] = key.split('_');
    condition[i][name] = data[key];

    this.setState({activities: this.activities});
  }

  setActivityInfo(index, data) {

    // Use the index information to find the activity
    const activity = this.props.activities.find(item => item.id === index.aid);

    Object.assign(activity, data);
    this.setState({activities: this.activities});
  }

  setProcessInfo(index, value) {
    Object.assign(this.state.meta, value);
    this.setState({meta: this.state.meta});
  }

}

Process.defaultProps = {
  emitter: undefined,
  activities: [
    {
      id: '00000001',
      name: 'Start',
      transitions: [{id: '00000001', to: '99999999'}],
      type: 'start'
    },
    {
      id: '99999999',
      name: 'End',
      transitions: [{id: '99999999', to: ''}],
      type: 'end'
    }
  ],
  meta: {
    name: '',
    description: '',
    fields: '',
    header: {}
  }
};

Process.init = (node, option) => {
  return ReactDOM.render(
    React.createElement(Process, option),
    document.getElementById(node)
  );
};
